// 
// 
// RH 74% Dew -2  Set to -6

#include "Defrost.h"
#include "HPController.h"

#define MINDEFROSTTIME	120
#define STEP1	30					//All off (for 30)
#define STEP2	250					//Compressor on (from 30 to 250)
#define STEP3	390					//compressor off (from 250 to 390)
#define STEP4	420					//Fan + valve on (390 to 420)

extern Ticker tickerDefrost;
void DefrostClass::defrostCheck() {
	char buf[10];

	lastDefrostStep = defrostStep;

	//Temperature needs to be < defrost temperature for > defrostStartDelay seconds or we cancel the defrost
	if (defrostTimer < defrostDelay)
	{
		defrostStep = 0;													//Wait for defrost
		defrostTimeToNextStep = defrostDelay;
		if (TEMP_EVAPORATOR > defrostTemperature)
		{
			DEBUG_PRINTLN(G("Cancel defrost"));
			tickerDefrost.detach();
			defrostFlag = false;
			defrostTimer = 0;
		}
	}
	else if (defrostTimer <= STEP1 + defrostDelay)
	{
		defrostStep = 1;													//Valve + compressor + fan off
		defrostTimeToNextStep = STEP1 + defrostDelay;
	}
	else if (defrostTimer <= STEP2 + defrostDelay)
	{
		defrostStep = 2;													//Compressor on
		defrostTimeToNextStep = STEP2 + defrostDelay;

		/*Check minimum defrost time has been reached*/
		if (TEMP_EVAPORATOR >= defrostUpperTemperature)
		{
			DEBUG_PRINTLN(G("Upper defrost temperature reached, cancel STEP2"));
			defrostTimer = STEP2 + defrostDelay;							//Cancel defrost STEP2 if we have reached the programmed upper temperature
			defrostStep = 3;												//Compressor off
		}
	}
	else if (defrostTimer <= STEP3 + defrostDelay)
	{
		defrostStep = 3;													//Compressor off
		defrostTimeToNextStep = STEP3 + defrostDelay;
	}
	else if (defrostTimer <= STEP4 + defrostDelay)
	{
		defrostStep = 4;													//Valve + fan on
		defrostTimeToNextStep = STEP4 + defrostDelay;
	}
	else																	//End defrost
	{
		defrostStep = 5;
		tickerDefrost.detach();
		defrostFlag = false;
		defrostTimer = 0;
		lastDefrost = millis();												//Save the last defrost end time
		lastDefrostTime = ntp.getLocalTime();
		ss.sendWsMessage(lastDefrostTime, "lastDefrost");					//Update webpage with last defrost time
		DEBUG_PRINTLN(G("Defrost ended"));
	}

	if (defrostStep != lastDefrostStep)
	{
		DEBUG_PRINTF(G("Defrost step %d\n"), defrostStep);
		snprintf(buf, sizeof(buf), "%d", defrostStep);
		ss.sendWsMessage(buf, "defStep");
	}

	DEBUG_PRINTF(G("defrostCount: %d, step:%d\n"), defrostTimer, defrostStep);
	snprintf(buf, sizeof(buf), "%d", defrostTimer);
	ss.sendWsMessage(buf, "defrost");

	//If defrost step has changed then set relays accordingly
	if (lastDefrostStep != defrostStep) {
		switch (defrostStep)
		{
		case 1:
			ss.setRelay(COMPRESSOR, OFF);
			ss.bValveOff = true;//  setRelay(VALVE, OFF);
			ss.setRelay(FAN, OFF);
			break;
		case 2:
			ss.setRelay(COMPRESSOR, ON);
			break;
		case 3:
			ss.setRelay(COMPRESSOR, OFF);
			break;
		case 4:
			ss.setRelay(FAN, ON);
			ss.setRelay(VALVE, ON);
			break;
		case 5:
			if((ss.fanState == ON)&&(ss.valveState == ON))
				ss.setRelay(COMPRESSOR, ON);
			defrostStep = 0;
			break;
		}
		lastDefrostStep = defrostStep;
	}
}

//void DefrostClass::defrostCheck() {
//	char buf[10];
//
//	lastDefrostStep = defrostStep;
//	timeToRunDefrost = (defrostRuntime * 60) + defrostDelay;		//Defrost time if not overridden
//
//	//Temperature needs to be < defrost temperature for > defrostStartDelay seconds or we cancel the defrost
//	if (defrostTimer < defrostDelay)
//	{
//		if (TEMP_EVAPORATOR > defrostTemperature)
//		{
//			DEBUG_PRINTLN(G("Cancel defrost"));
//			tickerDefrost.detach();
//			defrostFlag = false;
//			defrostTimer = defrostRuntime * 60;
//		}
//		return;
//	}
//
//	else if (defrostTimer <= STEP1 + defrostDelay)
//	{
//		defrostStep = 1;													//Valve + compressor + fan off
//		defrostTimeToNextStep = STEP1 + defrostDelay;
//	}
//	else if (defrostTimer <= timeToRunDefrost)
//	{
//		defrostStep = 2;													//Compressor on
//		defrostTimeToNextStep = timeToRunDefrost;
//
//		/*Check minimum defrost time has been reached*/
//		if ((TEMP_EVAPORATOR >= defrostUpperTemperature) && (defrostTimer > MINDEFROSTTIME))
//		{
//			DEBUG_PRINTLN(G("Upper defrost temperature reached, cancel defrost"));
//			defrostTimer = timeToRunDefrost;								//Cancel defrost if we have reached the programmed upper temperature
//			defrostStep = 3;												//Compressor off
//		}
//	}
//	else if (defrostTimer <= timeToRunDefrost + STEP2)
//	{
//		defrostStep = 3;													//Compressor off
//		defrostTimeToNextStep = timeToRunDefrost + STEP3;
//	}
//	else if (defrostTimer <= timeToRunDefrost + STEP3)
//	{
//		defrostStep = 4;													//Valve + fan on
//		defrostTimeToNextStep = timeToRunDefrost + STEP4;
//	}
//	else if (defrostTimer <= timeToRunDefrost + STEP5)
//	{
//		defrostTimeToNextStep = timeToRunDefrost + STEP5;
//		tickerDefrost.detach();
//		defrostFlag = false;
//		defrostTimer = defrostRuntime * 60;
//		lastDefrost = ntp.getLocal();										//Save the last defrost end time
//		lastDefrostTime = ntp.getLocalTime();
//		ss.sendWsMessage(lastDefrostTime, "lastDefrost");					//Update webpage with last defrost time
//		defrostStep = 5;
//		DEBUG_PRINTLN(G("Defrost ended"));
//	}
//
//	if (defrostStep != lastDefrostStep)
//	{
//		DEBUG_PRINTF(G("Defrost step %d\n"), defrostStep);
//		sprintf(buf, "%d", defrostStep);
//		ss.sendWsMessage(buf, "defStep");
//	}
//
//	DEBUG_PRINTF(G("defrostCount: %d, step:%d\n"), defrostTimer, defrostStep);
//	sprintf(buf, "%d", defrostTimer);
//	ss.sendWsMessage(buf, "defrost");
//
//	//If defrost step has changed then set relays accordingly
//	if (lastDefrostStep != defrostStep) {
//		switch (defrostStep)
//		{
//		case 1:
//			ss.setRelay(COMPRESSOR, OFF);
//			ss.bValveOff = true;//  setRelay(VALVE, OFF);
//			ss.setRelay(FAN, OFF);
//			break;
//		case 2:
//			ss.setRelay(COMPRESSOR, ON);
//			break;
//		case 3:
//			ss.setRelay(COMPRESSOR, OFF);
//			break;
//		case 4:
//			ss.setRelay(FAN, ON);
//			ss.setRelay(VALVE, ON);
//			break;
//		case 5:
//			ss.setRelay(COMPRESSOR, ON);
//			defrostStep = 0;
//			break;
//		}
//		lastDefrostStep = defrostStep;
//	}
//}

DefrostClass Defrost;

