// ntpTime.h
//Set up and use the SNTP service in the ESP8266
//Time is corrected with local time change for DST

/*    Configuration / use
#include <coredecls.h>                  // settimeofday_cb()
#include "ntpTime.h"

NtpTime ntp;
//Need these 2 calls to in setup to setup time
settimeofday_cb(time_is_set);
ntp.setupTime();

//This function in main cpp or ino to set the system time
void time_is_set(void)
{
	//Serial.println("------------------ settimeofday() was called ------------------");
	ntp.setSystemTime();
}
*/

#ifndef _NTPTIME_h
#define _NTPTIME_h

#include <Arduino.h> 
#include <Timezone.h>					//https://github.com/JChristensen/Timezone
#include <TimeLib.h>					// https://github.com/PaulStoffregen/Time
#include <time.h>						// time() ctime()
#include <sys/time.h>                   // struct timeval

class NtpTime
{
protected:
	TimeChangeRule myDST = { "DST", Last, Sun, Mar, 2, +120 };		//Daylight time = UTC - 1 hours
	TimeChangeRule mySTD = { "CET", Last, Sun, Oct, 3, +60 };     //Standard time = UTC - 2 hours
	TimeChangeRule* tcr;						//pointer to the time change rule, use to get TZ abbrev

 public:
	int dayT;
	int monthT;
	int year2;
	int yearT;
	int hourT;
	int minuteT;
	int secondT;
	bool time_set = false;
	time_t local;
	bool daytime = false;
	int dayHour = 6;				//Time day starts
	int nightHour = 21;				//Time night starts

	time_t getUTC();
	time_t getLocal();
	String getLocalTime();
	time_t getLocalTime_t();
	String getLocalDate();
	String getLocalDate(time_t tm);
	void setupTime();
	void setSystemTime();
	bool isTimeDaytime(time_t t);				//True for daytime, false for night
	time_t tmConvert_t(int YYYY, byte MM, byte DD, byte hh, byte mm, byte ss, bool local);
	String getFullTimeDate();

	String printTime(time_t t);
};

extern NtpTime ntp;

#endif

